﻿using System;
using System.Diagnostics;
using System.ComponentModel;
using System.Security.Cryptography;

namespace exc.jdbi.Randoms
{
  /// <summary>
  /// Simple Int64/Int32 Crypto-Random
  /// </summary>
  //[DebuggerNonUserCode]
  internal sealed class SOCR
  {
    /// <summary>
    /// Stellt sicher, dass die WarmUp-Phase nur
    /// einmal beim Starten des Programmes durch-
    /// laufen wird.
    /// </summary>
    private static bool FirstWarmUp = true;

    /// <summary>
    /// RandomNumberGenerator Variable.
    /// </summary>
    [Browsable(false)]
    [DebuggerBrowsable(DebuggerBrowsableState.Never)]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    public static RandomNumberGenerator _Rand = RandomNumberGenerator.Create();

    /// <summary>
    /// Fill a Array of Byte with Random-Bytes
    /// without Zero
    /// </summary>
    /// <param name="bytes">Byte-Array</param>
    internal static void NextBytes(byte[] bytes)
    {
      if (FirstWarmUp)
        WarmUp();

      _Rand.GetNonZeroBytes(bytes);
    }

    /// <summary>
    /// Next Random Int32 (without Exceptions)
    /// </summary>
    /// <returns>Return a Random Int32</returns>
    internal static int Next()
    {
      // Crypto-random number of Int32
      if (FirstWarmUp)
        WarmUp();

      int scale = -1;
      byte[] b = null;
      int sz = sizeof(int);
      while (scale < 0)
      {
        b = new byte[sz];
        _Rand.GetNonZeroBytes(b);
        scale = b[0] | b[1] << 8 | b[2] << 16 | b[3] << 24;
      }
      return Math.Abs(scale);
    }

    /// <summary>
    /// Random Int32 in Range min - max (without Exceptions)
    /// </summary>
    /// <param name="_min">Least legal value for the Random number.</param>
    /// <param name="_max">The greatest legal return value.</param>
    /// <returns>Return a Int32 [min - max]</returns>
    internal static int Next(int _min, int _max)
    {
      // Crypto-random number of Int32 with min - max
      if (_min == _max)
        return _min;
      SwapCheck(ref _min, ref _max);
      int scale = Next();
      return Math.Abs((int)(_min + (_max - _min) *
             (scale / (double)int.MaxValue)));
    }

    /// <summary>
    /// Next Random Int64 (without Exceptions)
    /// </summary>
    /// <returns>Return a Random Int64</returns>
    internal static long NextI64()
    {
      // Crypto-random number of Int64
      if (FirstWarmUp)
        WarmUp();
      long scale = -1;
      byte[] b = null;
      int sz = sizeof(long);
      while (scale < 0)
      {
        b = new byte[sz];
        _Rand.GetNonZeroBytes(b);
        scale = Convert.ToInt64(b[0]) | Convert.ToInt64(b[1]) << 8 | Convert.ToInt64(b[2]) << 16 | Convert.ToInt64(b[3]) << 24 |
                Convert.ToInt64(b[4]) << 32 | Convert.ToInt64(b[5]) << 40 | Convert.ToInt64(b[6]) << 48| Convert.ToInt64(b[7]) << 56;
      }
      return Math.Abs(scale);
    }

    /// <summary>
    /// Random Int64 in Range min - max (without Exceptions)
    /// </summary>
    /// <param name="_min">Least legal value for the Random number.</param>
    /// <param name="_max">The greatest legal return value.</param>
    /// <returns>Return a Int32 [min - max]</returns>
    internal static long NextI64(long _min, long _max)
    {
      // Crypto-random number of Int64 with min - max
      if (_min == _max)
        return _min;
      SwapCheck(ref _min, ref _max);
      long scale = NextI64();
      return Math.Abs((long)(_min + (_max - _min) *
             (scale / (double)long.MaxValue)));
    }

    /// <summary>
    /// Swaps two IntNum if necessary
    /// </summary>
    /// <typeparam name="T">Generic Datatyp</typeparam>
    /// <param name="_min">Least legal value.</param>
    /// <param name="_max">Greater legal value.</param>
    private static void SwapCheck<T>(ref T _min, ref T _max) where T :
      IComparable<T>
    {
      // Swaps two Variable of T
      if (_min.CompareTo(_max) > 0)
      {
        T tmp; tmp = _min;
        _min = _max; _max = tmp;
      }
    }

    /// <summary>
    /// Dient der Warmphase, und kommt
    /// beim Ersten Nutzen der Klasse
    /// zum Zug.
    /// </summary>
    private static void WarmUp()
    {
      FirstWarmUp = false;
      var startcount = NextI64() % 1000;
      for (int i = 0; i < startcount; i++)
        NextI64();
    }

    /// <summary>
    /// Private CTor
    /// </summary>
    private SOCR()
    {
    }
  }
}
